import { html, render } from "uhtml";
import {
  deriveSharedSecret,
  encrypt,
  retrieveOrGenerateKeyPair,
} from "../utils/crypto";

const ENCRYPTED_URL_INPUT_ID = "encrypted-url";
const MESSAGE_INPUT_ID = "message";
const ENCRYPT_BUTTON_ID = "encrypt";
const REQUEST_PUBLIC_KEY = "requestPublicKey";
const COPY_BUTTON_ID = "copy-button";

type PageState = {
  message: string;
  encryptedUrl: string;
  isCopyButtonDisabled: boolean;
  container: HTMLElement | null;
};
const state: PageState = {
  message: "",
  encryptedUrl: "",
  isCopyButtonDisabled: true,
  container: null,
};

const update = (element: HTMLElement) => {
  // Re-render the component whenever the state changes
  render(element, App(state));
};

const App = (state: PageState) => html`
  <details open>
    <summary>How To Use:</summary>
    <ol>
      <li>
        Enter the information you want to send back to the original requester
        into the text input below.
      </li>
      <li>
        Click on 'Generate Response'. This will create a new URL that contains
        your encrypted message.
      </li>
      <li>
        Send the newly generated URL back to the original requester. Only their
        browser will be able to decrypt the message.
      </li>
    </ol>
  </details>
  <form>
    <input
      type="text"
      name="message"
      .id=${MESSAGE_INPUT_ID}
      placeholder="Enter your message here..."
      aria-label="Messag
 to Encrypt"
      required
      value=${state.message}
      @input=${(e: Event) => {
        state.message = (e.target as HTMLInputElement).value;
        update(state.container!);
      }}
    />
    <input
      .id=${ENCRYPT_BUTTON_ID}
      type="submit"
      value="Generate Response"
      aria-label="Generate Response"
      @click=${encryptData}
    />
    <input
      type="text"
      name="encrypted-url"
      .id=${ENCRYPTED_URL_INPUT_ID}
      placeholder="URL with your encrypted response
 will appear here..."
      aria-label="Encrypted URL"
      value=${state.encryptedUrl}
    />
    <button
      aria-label="Copy to Clipboard"
      .id=${COPY_BUTTON_ID}
      @click=${copyToClipboard}
      disabled=${state.isCopyButtonDisabled}
    >
      Copy to Clipboard
    </button>
  </form>
`;

export async function setupSendPage(element: HTMLElement, key: string) {
  localStorage.setItem(REQUEST_PUBLIC_KEY, key);
  state.container = element;
  update(element);
}

async function copyToClipboard(event: Event) {
  event.preventDefault();
  await navigator.clipboard.writeText(state.encryptedUrl);
  alert("Copied to clipboard! Send this URL to the requester.");
}

async function encryptData(event: Event) {
  event.preventDefault();
  const key = localStorage.getItem(REQUEST_PUBLIC_KEY)!;

  // Parse the 'p' parameter to get publicA
  const publicAJwk = JSON.parse(atob(key));

  // Import publicA as a CryptoKey
  const publicA = await window.crypto.subtle.importKey(
    "jwk",
    publicAJwk,
    {
      name: "ECDH",
      namedCurve: "P-256",
    },
    false,
    []
  );

  const keyPairB = await retrieveOrGenerateKeyPair();

  // Derive the AES key from your private key and the recipient's public key
  const aesKey = await deriveSharedSecret(keyPairB.privateKey, publicA);

  // Encrypt the message input value using the AES key
  const { encryptedData, iv } = await encrypt(aesKey, state.message);

  const ecdhPublicJwk = await window.crypto.subtle.exportKey(
    "jwk",
    keyPairB.publicKey
  );
  const url = new URL(window.location.toString());
  url.search = "";
  url.hash = `p=${encodeURIComponent(
    btoa(JSON.stringify(ecdhPublicJwk))
  )}&iv=${encodeURIComponent(
    btoa(String.fromCharCode.apply(null, Array.from(iv)))
  )}&m=${encodeURIComponent(
    btoa(
      String.fromCharCode.apply(null, Array.from(new Uint8Array(encryptedData)))
    )
  )}`;

  state.encryptedUrl = url.toString();
  state.isCopyButtonDisabled = false;
  update(state.container!);
}
